@@ -2,7 +2,7 @@
 #' @keywords internal
 randomization_schema <- data.frame(
   schema = c("Pocock-Simon", "Permuted-Block", "Simple"),
-  id = c("ps", "pb", "sp"),
+  id = c("ps", "pb", "sr"),
   stringsAsFactors = FALSE
 )
 
@@ -28,7 +28,7 @@ h_get_vars <- function(treatment) {
   if (length(schema) > 1) {
     stop("only one randomization schema is allowed!")
   } else if (length(schema) == 0) {
-    schema <- "sp"
+    schema <- "sr"
   }
   strata <- setdiff(all.vars(treatment[[3]]), ".")
   list(
@@ -60,6 +60,7 @@ h_get_vars <- function(treatment) {
 #' - `treatment`: Name of the treatment variable.
 #' - `strata`: Name of the strata variable.
 #' - `schema`: Randomization schema.
+#' - `randomization_strata`: Names of the randomization strata variables.
 #' - `covariates`: Names of the covariates in the model.
 #' - `model`: A formula only including the covariates, but not treatment or strata variables.
 #' - `n_levels`: Number of treatment levels.
@@ -67,6 +68,7 @@ h_get_vars <- function(treatment) {
 #'
 #' @keywords internal
 h_prep_survival_input <- function(formula, data, treatment) {
+  # Process the treatment formula.
   trt_vars <- h_get_vars(treatment)
   assert_data_frame(data)
   assert_formula(formula)
@@ -82,8 +84,18 @@ h_prep_survival_input <- function(formula, data, treatment) {
   }
   trt_lvls <- levels(data[[trt_vars$treatment]])
   n_lvls <- length(trt_lvls)
-  covariates <- all.vars(formula[[3]])
-  assert_disjunct(c(trt_vars$treatment, trt_vars$strata), covariates)
+
+  # Process analysis model formula.
+  tms <- terms(formula, specials = "strata")
+  specials <- survival::untangle.specials(tms, "strata")
+  analysis_strata <- if (length(specials$vars) > 0) {
+    all.vars(as.formula(paste("~", paste(specials$vars, collapse = " + "))))
+  } else {
+    character()
+  }
+
+  covariates <- setdiff(all.vars(formula[[3]]), analysis_strata)
+  assert_disjunct(trt_vars$treatment, covariates)
 
   # Extract survival time and censoring indicator from the left hand side of the formula.
   lhs <- formula[[2]]
@@ -107,15 +119,21 @@ h_prep_survival_input <- function(formula, data, treatment) {
     stop("Left hand side of formula must be a Surv() object.")
   }
 
-  # Extract model without left hand side.
-  model <- as.formula(as.call(as.list(formula)[-2L]))
+  # Extract model without left hand side and without strata() terms.
+  covariates_formula <- if (length(specials$vars) > 0) {
+    stats::update(formula, as.formula(paste0("~ . -", paste(specials$vars, collapse = "-"))))
+  } else {
+    formula
+  }
+  model <- as.formula(as.call(as.list(covariates_formula)[-2L]))
 
   list(
     data = data,
     time = time_var,
     status = status_var,
     treatment = trt_vars$treatment,
-    strata = trt_vars$strata,
+    randomization_strata = trt_vars$strata,
+    strata = analysis_strata,
     schema = trt_vars$schema,
     covariates = covariates,
     model = model,
@@ -309,3 +327,47 @@ h_confint <- function(x, parm, level = 0.95, transform, include_se = FALSE, ...)
   }
   transform(ret)
 }
+
+#' Check Whether First Factor is Nested in Second Factor
+#'
+#' This function checks whether one factor variable is nested in another in terms of their levels
+#' and the mapping of observations to these levels:
+#'
+#' - Both factors must have the same length.
+#' - The positions of `NA` values must be identical in both factors.
+#' - After removing `NA` values, `f1` must have at least as many observed levels as `f2`.
+#' - The mapping of observations to levels must be consistent, meaning that each level in `f1`
+#'   corresponds to exactly one level in `f2`. On the other hand, multiple levels in `f1`
+#'   can map to the same level in `f2`.
+#'
+#' @param f1 A factor variable, supposed to be more fine grained than `f2`.
+#' @param f2 A factor variable, supposed to be more coarse grained than `f1`.
+#' @return `TRUE` if `f1` is nested in `f2`, `FALSE` otherwise.
+#'
+#' @keywords internal
+h_first_fct_nested_in_second <- function(f1, f2) {
+  assert_factor(f1)
+  assert_factor(f2)
+
+  if (length(f1) != length(f2)) {
+    return(FALSE)
+  }
+
+  f1_na <- which(is.na(f1))
+  f2_na <- which(is.na(f2))
+  if (!identical(f1_na, f2_na)) {
+    return(FALSE)
+  }
+
+  f1 <- droplevels(stats::na.omit(f1))
+  f2 <- droplevels(stats::na.omit(f2))
+  if (nlevels(f1) < nlevels(f2)) {
+    return(FALSE)
+  }
+
+  f1_int <- as.integer(f1)
+  f2_int <- as.integer(f2)
+  f1_f2_int <- paste(f1_int, f2_int, sep = "-")
+  unique_combinations <- unique(f1_f2_int)
+  length(unique_combinations) == nlevels(f1)
+}
