@@ -17,7 +17,8 @@ test_that("h_log_hr_est_via_score does not give spurious warning", {
     time = "time",
     status = "status",
     strata = "sex",
-    model = ~ age + meal.cal + wt.loss
+    model = ~ age + meal.cal + wt.loss,
+    randomization_strata = character()
   ))
   expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
 })
@@ -29,7 +30,8 @@ test_that("h_log_hr_est_via_score extends the search interval as needed", {
     df = surv_data,
     treatment = "sex",
     time = "time",
-    status = "status"
+    status = "status",
+    randomization_strata = character()
   )
   expect_true(result$theta > 0.2)
   expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
@@ -47,6 +49,26 @@ test_that("h_lr_test_via_score works as expected", {
 })
 
 test_that("robin_surv_comparison works as expected without covariate adjustment", {
+  input <- h_prep_survival_input(
+    formula = survival::Surv(time, status) ~ 1,
+    data = surv_data,
+    treatment = sex ~ sr(1)
+  )
+  result <- robin_surv_comparison(
+    score_fun = h_lr_score_no_strata_no_cov,
+    vars = input,
+    data = input$data,
+    exp_level = 2,
+    control_level = 1,
+    contrast = "hazardratio",
+    treatment = input$treatment,
+    time = input$time,
+    status = input$status
+  )
+  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
+})
+
+test_that("robin_surv_comparison can skip the hazard ratio estimation", {
   input <- h_prep_survival_input(
     formula = survival::Surv(time, status) ~ 1,
     data = surv_data,
@@ -58,6 +80,7 @@ test_that("robin_surv_comparison works as expected without covariate adjustment"
     data = input$data,
     exp_level = 2,
     control_level = 1,
+    contrast = "none",
     treatment = input$treatment,
     time = input$time,
     status = input$status
@@ -69,13 +92,14 @@ test_that("robin_surv_no_strata_no_cov works as expected", {
   input <- h_prep_survival_input(
     formula = survival::Surv(time, status) ~ 1,
     data = surv_data,
-    treatment = sex ~ 1
+    treatment = sex ~ sr(1)
   )
   result <- robin_surv_no_strata_no_cov(
     vars = input,
     data = input$data,
     exp_level = 1,
-    control_level = 2
+    control_level = 2,
+    contrast = "hazardratio"
   )
   expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
 })
@@ -84,14 +108,15 @@ test_that("robin_surv_no_strata_no_cov gives the same results as RobinCar functi
   input <- h_prep_survival_input(
     formula = survival::Surv(time, status) ~ 1,
     data = surv_data,
-    treatment = ecog ~ 1
+    treatment = ecog ~ sr(1)
   )
   input$data <- na.omit(input$data)
   result <- robin_surv_no_strata_no_cov(
     vars = input,
     data = input$data,
     exp_level = 2,
-    control_level = 1
+    control_level = 1,
+    contrast = "hazardratio"
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
   # `tests-raw/test-survival.R`.
@@ -109,58 +134,62 @@ test_that("robin_surv_no_strata_no_cov gives the same results as RobinCar functi
 
 test_that("robin_surv_strata works as expected", {
   input <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ 1,
+    formula = survival::Surv(time, status) ~ 1 + strata(strata),
     data = surv_data,
-    treatment = sex ~ strata
+    treatment = sex ~ sr(1)
   )
   result <- robin_surv_strata(
     vars = input,
     data = input$data,
     exp_level = 1,
-    control_level = 2
+    control_level = 2,
+    contrast = "hazardratio"
   )
   expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
 })
 
 test_that("robin_surv_strata works with multiple strata variables", {
   input <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ 1,
+    formula = survival::Surv(time, status) ~ strata(strata, ecog),
     data = surv_data,
-    treatment = sex ~ strata + ecog
+    treatment = sex ~ sr(1)
   )
   result <- robin_surv_strata(
     vars = input,
     data = input$data,
     exp_level = 1,
-    control_level = 2
+    control_level = 2,
+    contrast = "hazardratio"
   )
   surv_data$strata_ecog <- interaction(surv_data$strata, surv_data$ecog, drop = TRUE)
   input2 <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ 1,
+    formula = survival::Surv(time, status) ~ strata(strata_ecog),
     data = surv_data,
-    treatment = sex ~ strata_ecog
+    treatment = sex ~ sr(1)
   )
   result2 <- robin_surv_strata(
     vars = input2,
     data = input2$data,
     exp_level = 1,
-    control_level = 2
+    control_level = 2,
+    contrast = "hazardratio"
   )
   expect_equal(result, result2, ignore_formula_env = TRUE)
 })
 
 test_that("robin_surv_strata gives the same results as RobinCar functions", {
   input <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ 1,
+    formula = survival::Surv(time, status) ~ strata(sex),
     data = surv_data,
-    treatment = ecog ~ sex
+    treatment = ecog ~ sr(1)
   )
   input$data <- na.omit(input$data)
   result <- robin_surv_strata(
     vars = input,
     data = input$data,
     exp_level = 2,
-    control_level = 1
+    control_level = 1,
+    contrast = "hazardratio"
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
   # `tests-raw/test-survival.R`.
@@ -180,13 +209,14 @@ test_that("robin_surv_cov works as expected", {
   input <- h_prep_survival_input(
     formula = survival::Surv(time, status) ~ age,
     data = surv_data,
-    treatment = sex ~ 1
+    treatment = sex ~ sr(1)
   )
   result <- robin_surv_cov(
     vars = input,
     data = input$data,
     exp_level = 1,
-    control_level = 2
+    control_level = 2,
+    contrast = "hazardratio"
   )
   expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
 })
@@ -195,7 +225,7 @@ test_that("robin_surv_cov gives the same results as RobinCar functions", {
   input <- h_prep_survival_input(
     formula = survival::Surv(time, status) ~ age,
     data = surv_data,
-    treatment = ecog ~ 1
+    treatment = ecog ~ sr(1)
   )
   input$data <- na.omit(input$data)
   result <- robin_surv_cov(
@@ -203,6 +233,7 @@ test_that("robin_surv_cov gives the same results as RobinCar functions", {
     data = input$data,
     exp_level = 2,
     control_level = 1,
+    contrast = "hazardratio",
     hr_se_plugin_adjusted = FALSE # To get the exact match with RobinCar.
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
@@ -221,51 +252,54 @@ test_that("robin_surv_cov gives the same results as RobinCar functions", {
 
 test_that("robin_surv_strata_cov works as expected", {
   input <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ age,
+    formula = survival::Surv(time, status) ~ age + strata(sex),
     data = surv_data,
-    treatment = ecog ~ sex
+    treatment = ecog ~ sr(1)
   )
   result <- robin_surv_strata_cov(
     vars = input,
     data = input$data,
     exp_level = 2,
-    control_level = 1
+    control_level = 1,
+    contrast = "hazardratio"
   )
   expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
 })
 
 test_that("robin_surv_strata_cov works with multiple strata variables", {
   input <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ age,
+    formula = survival::Surv(time, status) ~ age + strata(strata, ecog),
     data = surv_data,
-    treatment = sex ~ strata + ecog
+    treatment = sex ~ sr(1)
   )
   result <- robin_surv_strata_cov(
     vars = input,
     data = input$data,
     exp_level = 2,
-    control_level = 1
+    control_level = 1,
+    contrast = "hazardratio"
   )
   surv_data$strata_ecog <- interaction(surv_data$strata, surv_data$ecog, drop = TRUE)
   input2 <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ age,
+    formula = survival::Surv(time, status) ~ age + strata(strata_ecog),
     data = surv_data,
-    treatment = sex ~ strata_ecog
+    treatment = sex ~ sr(1)
   )
   result2 <- robin_surv_strata_cov(
     vars = input2,
     data = input2$data,
     exp_level = 2,
-    control_level = 1
+    control_level = 1,
+    contrast = "hazardratio"
   )
   expect_equal(result, result2, ignore_formula_env = TRUE)
 })
 
 test_that("robin_surv_strata_cov gives the same results as RobinCar functions", {
   input <- h_prep_survival_input(
-    formula = survival::Surv(time, status) ~ age,
+    formula = survival::Surv(time, status) ~ age + strata(sex),
     data = surv_data,
-    treatment = ecog ~ sex
+    treatment = ecog ~ sr(1)
   )
   input$data <- na.omit(input$data)
   result <- robin_surv_strata_cov(
@@ -273,6 +307,7 @@ test_that("robin_surv_strata_cov gives the same results as RobinCar functions",
     data = input$data,
     exp_level = 2,
     control_level = 1,
+    contrast = "hazardratio",
     hr_se_plugin_adjusted = FALSE # To get the exact match with RobinCar.
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
@@ -383,7 +418,7 @@ test_that("robin_surv works as expected without strata or covariates", {
   result <- robin_surv(
     Surv(time, status) ~ 1,
     data = surv_data,
-    treatment = ecog ~ 1
+    treatment = ecog ~ sr(1)
   )
   expect_s3_class(result, "surv_effect")
   expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
@@ -395,7 +430,7 @@ test_that("robin_surv gives the same results as RobinCar functions without strat
   result <- robin_surv(
     Surv(time, status) ~ 1,
     data = na.omit(surv_data),
-    treatment = ecog ~ 1
+    treatment = ecog ~ sr(1)
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
   # `tests-raw/test-survival.R`.
@@ -414,9 +449,9 @@ test_that("robin_surv gives the same results as RobinCar functions without strat
 
 test_that("robin_surv works as expected with strata", {
   result <- robin_surv(
-    Surv(time, status) ~ 1,
+    Surv(time, status) ~ 1 + strata(strata),
     data = surv_data,
-    treatment = sex ~ strata
+    treatment = sex ~ sr(1)
   )
   expect_s3_class(result, "surv_effect")
   expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
@@ -426,9 +461,9 @@ test_that("robin_surv works as expected with strata", {
 
 test_that("robin_surv gives the same results as RobinCar functions with strata", {
   result <- robin_surv(
-    Surv(time, status) ~ 1,
+    Surv(time, status) ~ 1 + strata(sex),
     data = na.omit(surv_data),
-    treatment = ecog ~ sex
+    treatment = ecog ~ sr(1)
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
   # `tests-raw/test-survival.R`.
@@ -449,7 +484,7 @@ test_that("robin_surv works as expected with covariates", {
   result <- robin_surv(
     Surv(time, status) ~ age,
     data = surv_data,
-    treatment = ecog ~ 1
+    treatment = ecog ~ sr(1)
   )
   expect_s3_class(result, "surv_effect")
   expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
@@ -461,7 +496,7 @@ test_that("robin_surv gives the same results as RobinCar functions with covariat
   result <- robin_surv(
     Surv(time, status) ~ age,
     data = na.omit(surv_data),
-    treatment = ecog ~ 1
+    treatment = ecog ~ sr(1)
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
   # `tests-raw/test-survival.R`.
@@ -482,7 +517,7 @@ test_that("robin_surv gives the same results as RobinCar for single factor covar
   result <- robin_surv(
     Surv(time, status) ~ sex,
     data = na.omit(surv_data),
-    treatment = ecog ~ 1,
+    treatment = ecog ~ sr(1),
     hr_se_plugin_adjusted = FALSE
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
@@ -502,9 +537,9 @@ test_that("robin_surv gives the same results as RobinCar for single factor covar
 
 test_that("robin_surv works as expected with strata and covariates", {
   result <- robin_surv(
-    Surv(time, status) ~ age + ph.karno,
+    Surv(time, status) ~ age + ph.karno + strata(sex),
     data = surv_data,
-    treatment = ecog ~ sex
+    treatment = ecog ~ sr(1)
   )
   expect_s3_class(result, "surv_effect")
   expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
@@ -514,9 +549,9 @@ test_that("robin_surv works as expected with strata and covariates", {
 
 test_that("robin_surv gives the same results as RobinCar functions with strata and covariates", {
   result <- robin_surv(
-    Surv(time, status) ~ age,
+    Surv(time, status) ~ age + strata(sex),
     data = na.omit(surv_data),
-    treatment = ecog ~ sex
+    treatment = ecog ~ sr(1)
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
   # `tests-raw/test-survival.R`.
@@ -537,7 +572,7 @@ test_that("robin_surv also works with multiple pairwise comparisons", {
   result <- robin_surv(
     Surv(time, status) ~ 1,
     data = surv_data,
-    treatment = strata ~ 1
+    treatment = strata ~ sr(1)
   )
   expect_s3_class(result, "surv_effect")
   comparisons <- c("1 v.s. 0", "2 v.s. 0", "3 v.s. 0", "2 v.s. 1", "3 v.s. 1", "3 v.s. 2")
@@ -551,7 +586,7 @@ test_that("robin_surv allows the user to optionally define the comparisons of in
   result <- robin_surv(
     Surv(time, status) ~ 1,
     data = surv_data,
-    treatment = strata ~ 1,
+    treatment = strata ~ sr(1),
     comparisons = list(c(1, 2), c(3, 3))
   )
   expect_s3_class(result, "surv_effect")
@@ -566,13 +601,13 @@ test_that("robin_surv allows to use unadjusted standard error", {
   result <- robin_surv(
     Surv(time, status) ~ age,
     data = surv_data,
-    treatment = ecog ~ 1,
+    treatment = ecog ~ sr(1),
     hr_se_plugin_adjusted = FALSE
   )
   result_adjusted <- robin_surv(
     Surv(time, status) ~ age,
     data = surv_data,
-    treatment = ecog ~ 1,
+    treatment = ecog ~ sr(1),
     hr_se_plugin_adjusted = TRUE
   )
   # Only the standard error should differ.
@@ -585,7 +620,7 @@ test_that("robin_surv allows to use unadjusted standard error", {
 test_that("robin_surv gives the same results as RobinCar for strong correlation covariate with strata", {
   set.seed(2040)
   surv_data2 <- surv_data
-  surv_data2$ecog <- as.factor(ifelse(
+  surv_data2$ecog <- factor(ifelse(
     surv_data$strata == 1,
     rbinom(nrow(surv_data), 1, prob = 0.2),
     surv_data$ecog
@@ -593,9 +628,9 @@ test_that("robin_surv gives the same results as RobinCar for strong correlation
   surv_data2 <- surv_data2[surv_data2$strata %in% c(0, 1), ]
 
   result <- robin_surv(
-    formula = Surv(time, status) ~ ecog,
+    formula = Surv(time, status) ~ ecog + strata(strata),
     data = na.omit(surv_data2),
-    treatment = sex ~ strata,
+    treatment = sex ~ sr(1),
     hr_se_plugin_adjusted = FALSE
   )
   # These values are extracted from RobinCar (version 1.0.0) results, see
@@ -612,3 +647,177 @@ test_that("robin_surv gives the same results as RobinCar for strong correlation
   expect_equal(result$log_hr_coef_mat[, "Estimate"], robincar_result$estimate, tolerance = 1e-1)
   expect_equal(result$log_hr_coef_mat[, "Std.Err"], robincar_result$se, tolerance = 1e-2)
 })
+
+test_that("robin_surv works also with character variable in the correlation case", {
+  set.seed(2040)
+  surv_data2 <- surv_data
+  surv_data2$ecog <- as.character(ifelse(
+    surv_data$strata == 1,
+    rbinom(nrow(surv_data), 1, prob = 0.2),
+    surv_data$ecog
+  ))
+  surv_data2 <- surv_data2[surv_data2$strata %in% c(0, 1), ]
+  # Here we only have ecog 1 for strata 0:
+  assert_true(all(surv_data2$ecog[surv_data2$strata == 0] == "1"))
+
+  result <- expect_silent(robin_surv(
+    formula = Surv(time, status) ~ ecog + strata(strata),
+    data = na.omit(surv_data2),
+    treatment = sex ~ sr(1),
+    hr_se_plugin_adjusted = FALSE
+  ))
+  expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-2, style = "deparse")
+})
+
+test_that("robin_surv can skip the hazard ratio estimation", {
+  result <- robin_surv(
+    Surv(time, status) ~ age,
+    data = surv_data,
+    treatment = ecog ~ 1,
+    contrast = "none"
+  )
+  expect_identical(result$contrast, "none")
+  expect_scalar_na(result$estimate)
+  expect_scalar_na(result$se)
+  expect_scalar_na(result$hr_n)
+  expect_scalar_na(result$hr_sigma_l2)
+  expect_null(result$log_hr_coef_mat)
+})
+
+test_that("robin_surv can work with interaction covariates", {
+  result <- expect_silent(robin_surv(
+    Surv(time, status) ~ age + interaction(I(ph.karno < 80), I(wt.loss > 7)) + strata(ecog),
+    data = surv_data,
+    treatment = sex ~ sr(1)
+  ))
+  expect_s3_class(result, "surv_effect")
+})
+
+test_that("robin_surv works in a case where previously separate design matrices led to failure", {
+  result <- robin_surv(
+    Surv(time, status) ~ age + interaction(I(ph.karno < 50), I(wt.loss > 10)) + strata(strata),
+    data = surv_data,
+    treatment = sex ~ sr(1)
+  )
+  expect_s3_class(result, "surv_effect")
+})
+
+test_that("robin_surv gives a warning if stratified randomization was specified but simple log rank test used", {
+  expect_snapshot(
+    robin_surv(
+      Surv(time, status) ~ 1,
+      data = surv_data,
+      treatment = sex ~ pb(ecog)
+    )
+  )
+})
+
+test_that("robin_surv does not give a warning if strata are sufficiently included in the analysis model", {
+  full_strata <- with(surv_data, interaction(strata, ecog, drop = TRUE))
+  incl_strata <- surv_data$strata
+  assert_true(h_first_fct_nested_in_second(incl_strata, full_strata))
+
+  result <- expect_silent(robin_surv(
+    Surv(time, status) ~ 1 + strata(strata),
+    data = surv_data,
+    treatment = sex ~ pb(ecog, strata)
+  ))
+})
+
+test_that("robin_surv does give a warning if strata are not sufficiently included in the analysis model", {
+  full_strata <- surv_data$strata
+  incl_strata <- surv_data$ecog
+  assert_false(h_first_fct_nested_in_second(incl_strata, full_strata))
+
+  expect_snapshot(
+    robin_surv(
+      Surv(time, status) ~ 1 + strata(ecog),
+      data = surv_data,
+      treatment = sex ~ pb(strata)
+    )
+  )
+  expect_warning(
+    robin_surv(
+      Surv(time, status) ~ 1 + strata(ecog),
+      data = surv_data,
+      treatment = sex ~ pb(strata, ecog)
+    ),
+    "adjust for all joint levels in your `formula` using `+ strata` or",
+    fixed = TRUE
+  )
+  result <- expect_silent(robin_surv(
+    Surv(time, status) ~ 1 + strata(ecog) + strata(strata),
+    data = surv_data,
+    treatment = sex ~ pb(strata)
+  ))
+})
+
+test_that("robin_surv does give a warning if strata are not sufficiently included in the covariates", {
+  expect_snapshot(
+    robin_surv(
+      Surv(time, status) ~ 1 + ph.karno,
+      data = surv_data,
+      treatment = sex ~ pb(ecog)
+    )
+  )
+  result <- expect_silent(robin_surv(
+    Surv(time, status) ~ 1 + ph.karno + ecog,
+    data = surv_data,
+    treatment = sex ~ pb(ecog)
+  ))
+
+  surv_data2 <- droplevels(surv_data[surv_data$strata != "3", ])
+  expect_warning(
+    robin_surv(
+      Surv(time, status) ~ 1 + ecog,
+      data = surv_data2,
+      treatment = sex ~ pb(strata, ecog)
+    ),
+    "adjust for all joint levels in your `formula` using `+ strata` or",
+    fixed = TRUE
+  )
+  result <- expect_silent(robin_surv(
+    Surv(time, status) ~ 1 + strata,
+    data = surv_data2,
+    treatment = sex ~ pb(ecog)
+  ))
+})
+
+test_that("robin_surv warns if strata are insufficiently included in covariate adjusted stratified model", {
+  expect_snapshot(
+    robin_surv(
+      Surv(time, status) ~ 1 + strata(ecog) + ph.karno,
+      data = surv_data,
+      treatment = sex ~ pb(strata)
+    )
+  )
+  expect_warning(
+    robin_surv(
+      Surv(time, status) ~ 1 + strata(ecog) + ph.karno,
+      data = surv_data,
+      treatment = sex ~ pb(strata, ecog)
+    ),
+    "adjust for all joint levels in your `formula` using `+ strata` or",
+    fixed = TRUE
+  )
+  result <- expect_silent(robin_surv(
+    Surv(time, status) ~ 1 + strata(strata) + ph.karno,
+    data = surv_data,
+    treatment = sex ~ pb(strata)
+  ))
+})
+
+test_that("robin_surv only gives a single warning for randomization strata with multiple comparisons", {
+  expect_warning(
+    result <- robin_surv(
+      Surv(time, status) ~ 1,
+      data = surv_data,
+      treatment = strata ~ pb(ecog),
+    ),
+    "It looks like you have not included all of the variables that were used during randomization"
+  )
+  expect_s3_class(result, "surv_effect")
+  comparisons <- c("1 v.s. 0", "2 v.s. 0", "3 v.s. 0", "2 v.s. 1", "3 v.s. 1", "3 v.s. 2")
+  expect_matrix(result$log_hr_coef_mat, ncol = 4, nrow = 6)
+  expect_names(rownames(result$log_hr_coef_mat), identical.to = comparisons)
+})
