@@ -7,6 +7,7 @@ test_that("h_derived_outcome_vals works as expected", {
     time = "time",
     status = "status",
     covariates = c("age", "ph.karno"),
+    randomization_strata = character(),
     n = 400
   )
   expect_data_frame(result, ncol = 4 + 2 + 1, nrow = nrow(surv_data_full))
@@ -39,11 +40,15 @@ test_that("h_strat_derived_outcome_vals works as expected", {
     time = "time",
     status = "status",
     strata = "strata",
-    covariates = c("age", "ph.karno")
+    covariates = c("age", "ph.karno"),
+    randomization_strata = character()
   )
-  expect_list(result, len = nlevels(surv_data_full$strata), types = "data.frame", any.missing = FALSE)
-  expect_names(names(result), identical.to = levels(surv_data_full$strata))
-  expect_identical(unname(sapply(result, nrow)), as.integer(table(surv_data_full$strata)))
+  expect_data_frame(result, nrow = nrow(surv_data_full))
+  expect_names(
+    names(result),
+    identical.to = c("index", "treatment", "time", "status", "O_hat", "age", "ph.karno", ".stratum")
+  )
+  expect_identical(as.integer(table(result$.stratum)), as.integer(table(surv_data_full$strata)))
 })
 
 test_that("h_strat_derived_outcome_vals works with multiple strata", {
@@ -55,13 +60,21 @@ test_that("h_strat_derived_outcome_vals works with multiple strata", {
     time = "time",
     status = "status",
     strata = c("strata", "ecog"),
-    covariates = c("age", "ph.karno")
+    covariates = c("age", "ph.karno"),
+    randomization_strata = "ecog"
+  )
+  expect_data_frame(result, nrow = nrow(surv_data_full))
+  expect_names(
+    names(result),
+    identical.to = c("index", "treatment", "time", "status", "O_hat", "age", "ph.karno", "ecog", ".stratum")
   )
-  expect_list(result, len = 4L, types = "data.frame", any.missing = FALSE)
-  expect_names(names(result), identical.to = c("0.0", "2.0", "3.0", "1.1"))
   expect_identical(
-    unname(sapply(result, nrow)),
-    as.integer(table(interaction(surv_data_full$strata, surv_data_full$ecog, drop = TRUE)))
+    as.integer(table(result$.stratum)),
+    as.integer(table(interaction(
+      surv_data_full$strata,
+      surv_data_full$ecog,
+      drop = TRUE
+    )))
   )
 })
 
@@ -119,36 +132,40 @@ test_that("h_get_lm_input works correctly across treatment levels with character
 
 test_that("h_get_strat_lm_input works as expected", {
   set.seed(941)
-  df_split <- list(
-    "stratum1" = data.frame(
+  df_with_stratum <- rbind(
+    data.frame(
       index = 1:3,
       treatment = factor(c(0, 1, 0), labels = c("A", "B")),
       covariate1 = rnorm(3),
       covariate2 = rnorm(3),
-      O_hat = rnorm(3)
+      O_hat = rnorm(3),
+      .stratum = 1
     ),
-    "stratum2" = data.frame(
+    data.frame(
       index = 4:5,
       treatment = factor(c(1, 0), labels = c("A", "B")),
       covariate1 = rnorm(2),
       covariate2 = rnorm(2),
-      O_hat = rnorm(2)
+      O_hat = rnorm(2),
+      .stratum = 2
     )
   )
-  result <- h_get_strat_lm_input(df_split, model = ~ covariate1 + covariate2)
+  result <- h_get_strat_lm_input(df_with_stratum, model = ~ covariate1 + covariate2)
   expect_list(result, len = 2L)
-  expect_names(names(result), identical.to = c("stratum1", "stratum2"))
-  # Check first stratum, treatment group A:
-  expect_list(result[["stratum1"]][["A"]], len = 2L)
-  expect_matrix(result[["stratum1"]][["A"]][["X"]], ncol = 2L, nrow = 2L)
-  expect_numeric(result[["stratum1"]][["A"]][["y"]], len = 2L)
-  # Check first stratum, treatment group B:
-  expect_list(result[["stratum2"]][["B"]], len = 2L)
-  expect_matrix(result[["stratum2"]][["B"]][["X"]], ncol = 2L, nrow = 1L)
-  expect_numeric(result[["stratum2"]][["B"]][["y"]], len = 1L)
+  expect_names(names(result), identical.to = c("A", "B"))
+  # Check treatment group A:
+  expect_list(result[["A"]], len = 2L)
+  expect_matrix(result[["A"]][["X"]], ncol = 3L, nrow = 3L)
+  expect_subset(".stratum", colnames(result[["A"]][["X"]]))
+  expect_numeric(result[["A"]][["y"]], len = 3L)
+  # Check treatment group B:
+  expect_list(result[["B"]], len = 2L)
+  expect_matrix(result[["B"]][["X"]], ncol = 3L, nrow = 2L)
+  expect_subset(".stratum", colnames(result[["B"]][["X"]]))
+  expect_numeric(result[["B"]][["y"]], len = 2L)
 })
 
-test_that("h_get_beta_estimates works as expected", {
+test_that("h_get_lm_results works as expected", {
   set.seed(941)
   nobs <- 10
   df <- data.frame(
@@ -158,30 +175,32 @@ test_that("h_get_beta_estimates works as expected", {
     O_hat = rnorm(nobs)
   )
   lm_input <- h_get_lm_input(df, model = ~ covariate1 + covariate2)
-  result <- h_get_beta_estimates(lm_input)
+  result <- h_get_lm_results(lm_input)
   expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
 })
 
-test_that("h_get_strat_beta_estimates works as expected", {
+test_that("h_get_strat_lm_results works as expected", {
   set.seed(941)
   nobs <- 10
-  df_split <- list(
-    "stratum1" = data.frame(
+  df_with_stratum <- rbind(
+    data.frame(
       treatment = factor(sample(c("A", "B"), nobs, replace = TRUE)),
       covariate1 = rnorm(nobs),
       covariate2 = rnorm(nobs),
-      O_hat = rnorm(nobs)
+      O_hat = rnorm(nobs),
+      .stratum = 1
     ),
-    "stratum2" = data.frame(
+    data.frame(
       treatment = factor(sample(c("A", "B"), nobs, replace = TRUE)),
       covariate1 = rnorm(nobs),
       covariate2 = rnorm(nobs),
-      O_hat = rnorm(nobs)
+      O_hat = rnorm(nobs),
+      .stratum = 2
     )
   )
-  strat_lm_input <- h_get_strat_lm_input(df_split, model = ~ covariate1 + covariate2)
-  result <- h_get_strat_beta_estimates(strat_lm_input)
+  strat_lm_input <- h_get_strat_lm_input(df_with_stratum, model = ~ covariate1 + covariate2)
+  result <- h_get_strat_lm_results(strat_lm_input)
   expect_list(result, len = 2L)
-  expect_names(names(result), identical.to = c("A", "B"))
+  expect_names(names(result), identical.to = c("beta_est", "residuals"))
   expect_snapshot_value(result, tolerance = 1e-4, style = "serialize")
 })
